/*
 * File: planet.h
 * Author: Douglas Selph
 * Maintained by: Robin Powell
 * $Id: planet.h,v 1.1 1997/10/30 19:16:37 rlpowell Exp $
 *   Extracted from the Machine archive
 * Purpose:
 */

struct planets
{
  char *name;
  float x, y;
  float size, res;
  int pop;
  float industry;
  char *produce;
  float cap;
  float mat;
  float col;
  int who;
  int set;
  int floater;		/* not particular about where we are in the array */
};

#ifdef DEF
struct planets *planet;
#else
extern struct planets *planet;
#endif

